<?php

namespace goodboyalex\php_components_pack\enums;

use goodboyalex\php_components_pack\traits\EnumExtensionsTrait;

/**
 * Перечисление типов сериализации массива объектов.
 *
 * @author Александр Бабаев
 * @package php_components_pack
 * @version 1.0
 * @since 1.1.1
 */
enum ObjectArraySerializeMethod: int
{
    // Подключаю расширение для Enum
    use EnumExtensionsTrait;

    /**
     * Сериализация массива объектов с помощью serialize.
     */
    case Serialize = 1;

    /**
     * Сериализация массива объектов с помощью json_encode.
     *
     * @warning Для корректной работы требуется наличие json_encode в php.ini
     * @warning Этот способ сериализации НЕ РЕКОМЕНДУЕТСЯ для массивов объектов, представляющих собой классы, так как
     *     при десериализации классы не будут созданы: вместо этого будет созданы объекты класса stdClass. Вместо этого
     *     следует использовать метод JsonEncodeWithToArray.
     */
    case JsonEncode = 2;

    /**
     * Сериализация массива объектов с помощью ToArray, который возвращает массив. Этот массив впоследствии
     * сериализуется с помощью json_encode. Десерилизация происходит в обратном порядке.
     */
    case JsonEncodeWithToArray = 3;
}