<?php

namespace goodboyalex\php_components_pack\tests\extensions;

use goodboyalex\php_components_pack\exceptions\TypeException;
use goodboyalex\php_components_pack\extensions\TypeExtension;
use goodboyalex\php_components_pack\tests\data\A;
use goodboyalex\php_components_pack\tests\data\B;
use goodboyalex\php_components_pack\tests\data\C;
use goodboyalex\php_components_pack\tests\data\D;
use PHPUnit\Framework\TestCase;

class TypeExtensionTest extends TestCase
{
    public function testFromArray ()
    {
        // Подготавливаем данные к тестированию
        $this->PrepareForTest();

        // Зададим имя файла
        $fileName = __DIR__ . "/class.txt";

        // Если файл не найден
        if (!file_exists($fileName))
            // - то завершаем работу
            die("The test data file could not be found or has not yet passed the ToArray test! / Файл с данными для тестирования не найден или не пройдено ещё тестирование ToArray!");

        // Считываем данные из файла и преобразуем в массив
        $array = json_decode(file_get_contents($fileName), true);

        // Преобразуем в объект
        try {
            $class = TypeExtension::FromArray($array, D::CLOSURE_FROM_ARRAY());
        }
        catch (TypeException $e) {
            // - если ошибка, то выводим её и завершаем работу
            die($e->getMessage());
        }

        // Проверяем
        $this->assertEquals(D::class, $class::class);
        $this->assertEquals(A::class, $class->a::class);
        $this->assertEquals(B::class, $class->b::class);
        $this->assertEquals(9876, $class->b->b);
        $this->assertEquals(C::class, $class->c::class);
        $this->assertEquals(54321, $class->c->intC);
        $this->assertEquals('test_string', $class->stringD);
    }

    private function PrepareForTest (): void
    {
        require_once __DIR__ . '/../../sources/exceptions/TypeException.php';
        require_once __DIR__ . '/../../sources/extensions/TypeExtension.php';
        require_once __DIR__ . '/../data/A.php';
        require_once __DIR__ . '/../data/B.php';
        require_once __DIR__ . '/../data/C.php';
        require_once __DIR__ . '/../data/D.php';
    }

    public function testToArray ()
    {
        // Подготавливаем данные к тестированию
        $this->PrepareForTest();

        // Создаём тестовый класс
        $class = new D ('test_string', 12345, true, new A("test_string_A", 6789, false),
            new B("test_string_B", 9876, "false"), new C("test_string_C", 54321, true));

        // Преобразуем в массив
        try {
            $array = TypeExtension::ToArray($class, D::CLOSURE_TO_ARRAY());
        }
        catch (TypeException $e) {
            // - если ошибка, то выводим её и завершаем работу
            die($e->getMessage());
        }

        // Сохраняем массив в файл (для тестирования FromArray)
        file_put_contents(__DIR__ . "/class.txt", json_encode($array, JSON_PRETTY_PRINT));

        // Проверяем
        $this->assertEquals(D::class, $array['type_class']);
        $this->assertEquals(A::class, $array['a']['type_class']);
        $this->assertEquals(B::class, $array['b']['type_class']);
        $this->assertEquals(9876, $array['b']['b']);
        $this->assertEquals(C::class, $array['c']['type_class']);
        $this->assertEquals('test_string', $array['stringD']);
        $this->assertEquals(12345, $array['intD']);
        $this->assertTrue($array['boolD']);
    }
}
